﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using TradeServer.ScriptingDriver.DataObjects;
using TradeServer.ScriptingDriver.Interfaces;
using TicTacTec.TA.Library;

namespace PshaisIndicators
{
    public class Calibrator : IIndicatorScript
    {
    	int ma_length1 = 5;
    	Core.MAType ma_type1 = Core.MAType.Ema;
    	int ma_length2 = 20;
    	Core.MAType ma_type2 = Core.MAType.Ema;
    	double profit_margin = 0.04;
    	double loss_margin = 0.02;
    	
    	double profit_level, loss_level;
    	double ma_value1, ma_value2;
		
    	int curDir = -1;
    	int prevDir = -1;
    	bool sold = false;
    	
    	int max_sell_triggers = 3;
    	int count = 0;
    	
        // Properties
        public string Name
        {
            get { return "pshai's Calibrator"; }
        }
        public int TicksBack
        {
            get { 
        		var ma1 = Core.MovingAverageLookback(ma_length1, ma_type1);
        		var ma2 = Core.MovingAverageLookback(ma_length2, ma_type2);
        		return Math.Max(ma1, ma2) + 5;
        	}
        }

        // Setting fields
        public List<ScriptParameter> GetParameters()
        {
            List<ScriptParameter> res = new List<ScriptParameter>()
            {
            	new ScriptParameter("MA1 Length", ScriptParameterType.Integer, ma_length1.ToString()),
            	new ScriptParameter("MA1 Type", ScriptParameterType.String, ma_type1.ToString()),
            	new ScriptParameter("MA2 Length", ScriptParameterType.Integer, ma_length2.ToString()),
            	new ScriptParameter("MA2 Type", ScriptParameterType.String, ma_type2.ToString()),
            	new ScriptParameter("Profit Margin", ScriptParameterType.Decimal, profit_margin.ToString()),
            	new ScriptParameter("Loss Margin", ScriptParameterType.Decimal, loss_margin.ToString())
            };
            
            return res;
        }

        public void SetParameters(Dictionary<string, object> parameters)
        {
        	ma_length1 = Convert.ToInt32(parameters["MA1 Length"]);
        	ma_type1 = ParseMAType(parameters["MA1 Type"].ToString());
        	ma_length2 = Convert.ToInt32(parameters["MA2 Length"]);
        	ma_type2 = ParseMAType(parameters["MA2 Type"].ToString());
        	profit_margin = ParseDouble(parameters["Profit Margin"].ToString());
        	loss_margin = ParseDouble(parameters["Loss Margin"].ToString());
        }

        // Chart lines
        public List<ChartRecord> GetChartLines()
        {
            List<ChartRecord> chartLines = new List<ChartRecord>
            {
                new ChartRecord() { ChartIndex = 0, Name = string.Format("{0}({1})", ma_type1, ma_length1), HexLineColor = "#FFFFFF" },
                new ChartRecord() { ChartIndex = 0, Name = string.Format("{0}({1})", ma_type2, ma_length2), HexLineColor = "#FFFF00" },
                new ChartRecord() { ChartIndex = 0, Name = "Profit Level", HexLineColor = "#00FF00" },
                new ChartRecord() { ChartIndex = 0, Name = "Loss Level", HexLineColor = "#FF0000" }
            };
            return chartLines;
        }
        public List<decimal> GetChartData()
        {
            List<decimal> chartData = new List<decimal>
            {
                Convert.ToDecimal(ma_value1),
                Convert.ToDecimal(ma_value2),
                Convert.ToDecimal(profit_level),
                Convert.ToDecimal(loss_level)
            };

            return chartData;
        }

        // This is called as the script is loaded the first time, aka the init
        public void Init()
        {
        }

        // This is the brain so the say, this is where the calculation is done and an output must be produced.
        public IndicatorResult GetResult(IndicatorContext context)
        {
            try
            {
                IPriceInstrument instrument = context.PriceInstrument; // Its pre-created for speed based on the "TicksBack" property field
                int position = instrument.Close.Length - (1 + context.Offset);
                
                var highs = instrument.High;
                var lows = instrument.Low;
                var opens = instrument.Open;
                var closes = instrument.Close;
                var volumes = instrument.Volume;
                
                var price = closes[position];
                
                int ma_begin1, ma_len1;
                var ma_out1 = new double[position];
                var ready1 = Core.MovingAverage(0, position, closes, ma_length1, ma_type1, out ma_begin1, out ma_len1, ma_out1) == Core.RetCode.Success;
                
                int ma_begin2, ma_len2;
                var ma_out2 = new double[position];
                var ready2 = Core.MovingAverage(0, position, closes, ma_length2, ma_type2, out ma_begin2, out ma_len2, ma_out2) == Core.RetCode.Success;
                
                
                
                // default result
                var result = IndicatorResult.StayNoPosition;
                
                if(ready1 && ready2 && ma_len1 > 0 && ma_len2 > 0)
                {
                	ma_value1 = ma_out1[ma_len1-1];
                	ma_value2 = ma_out2[ma_len2-1];
                	
                	switch(curDir)
                	{
                		case 1:
                			result = IndicatorResult.BuyLong;
                			
                			if(price >= profit_level)
                				ReAdjustLevels(price);
                			
                			// sell
		                	if( !sold && price < loss_level && count < max_sell_triggers )
		                	{
		                		count++;
		                		
		                		if(count >= max_sell_triggers)
		                		{
		                			curDir = -1;
		                			prevDir = curDir;
		                			sold = true;
		                		}
		                		
		                		result = IndicatorResult.SellShort;
		                	}
                			
                			break;
                			
                		case -1:
                			result = IndicatorResult.SellShort;
                			
                			if( ma_value1 > ma_value2 && !sold )
		                	{
		                		curDir = 1; // buying direction
		                		
		                		if(prevDir != curDir)
		                		{
		                			prevDir = curDir;
		                			ReAdjustLevels(price);
		                		}
		                		
		                		result = IndicatorResult.BuyLong;
		                	}
                			
                			if( sold && ma_value1 < ma_value2 )
		                	{
		                		count = 0;
		                		sold = false;
		                	}
                			break;
                			
                		default:
                			break;
                	}
                }
                
                // return result
                return result;
            }
            catch (Exception ex) // This try catch block is an highly recommend practice
            {
                context.Logger.LogToFile(string.Format("Exception detected: {0}", ex.Message));
            }

            return IndicatorResult.StayNoPosition; // Default output
        }
        
        void ReAdjustLevels(double price)
        {
        	profit_level = price * (1 + profit_margin);
        	loss_level = price * (1 - loss_margin);
        }
        
        public static Core.MAType ParseMAType(string value)
		{
			Core.MAType val;
			switch( value.ToLower() )
			{
				case "sma":
					val = Core.MAType.Sma;
					break;
               	case "ema":
					val = Core.MAType.Ema;
					break;
               	case "dema":
					val = Core.MAType.Dema;
					break;
               	case "tema":
					val = Core.MAType.Tema;
					break;
               	case "trima":
					val = Core.MAType.Trima;
					break;
               	case "t3":
					val = Core.MAType.T3;
					break;
               	case "mama":
					val = Core.MAType.Mama;
					break;
               	case "kama":
					val = Core.MAType.Kama;
					break;
				case "wma":
					val = Core.MAType.Wma;
					break;
				default:
					val = Core.MAType.Sma;
					break;
			}
			
			return val;
		}
        
        double ParseDouble(string obj)
		{
			var numStyle = System.Globalization.NumberStyles.Any;
			var cult = System.Globalization.CultureInfo.InvariantCulture;
			
			double output;
			try {
				double.TryParse(
					obj.Replace(',', '.'),
					numStyle,
					cult,
					out output);
			}
			catch
			{
				output = 0;
			}
			
			return output;
		}
    }
}
